package com.starbuds.server.api.channel.service.admin.impl;

import cn.hutool.core.util.NumberUtil;
import cn.hutool.crypto.SecureUtil;
import com.starbuds.server.api.channel.common.pojo.admin.ChannelAdminInfoVO;
import com.starbuds.server.api.channel.common.pojo.admin.ChannelDTO;
import com.starbuds.server.api.channel.common.pojo.db.ChannelAdminSecurityDO;
import com.starbuds.server.api.channel.common.pojo.db.ChannelAdminTokenDO;
import com.starbuds.server.api.channel.common.service.admin.ChAdminSecurityService;
import com.starbuds.server.api.channel.common.service.admin.ChAdminService;
import com.starbuds.server.api.channel.common.service.admin.ChChannelService;
import com.starbuds.server.api.channel.service.admin.dao.ChannelAdminSecurityDAO;
import com.starbuds.server.api.channel.service.admin.dao.ChannelAdminTokenDAO;
import com.starbuds.server.api.manage.common.pojo.admin.TokenParam;
import com.starbuds.server.api.manage.service.admin.provider.TokenProvider;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/1/16 14:09
 * @Description 安全业务实现
 */
@Slf4j
@Service
public class ChAdminSecurityServiceImpl implements ChAdminSecurityService {

    @Autowired
    ChannelAdminSecurityDAO adminSecurityDAO;

    @Autowired
    ChannelAdminTokenDAO channelAdminTokenDAO;

    @Resource
    ChAdminService chAdminService;

    @Resource
    ChChannelService chChannelService;

    /**
     * 根据账号查询
     *
     * @param account
     * @return
     */
    @Override
    public ChannelAdminSecurityDO getSecurityByAccount(String account) {
        return adminSecurityDAO.selectByAccount(account);
    }

    /**
     * 根据账号id查询
     *
     * @param adminId
     * @return
     */
    @Override
    public ChannelAdminSecurityDO getSecurityByAdminId(Long adminId) {
        return adminSecurityDAO.selectById(adminId);
    }

    /**
     * 密码检查
     *
     * @param password
     * @param dbPass
     * @return
     */
    @Override
    public boolean checkPass(String password, String dbPass) {
        return dbPass.equals(encryptPassword(password));
    }

    /**
     * 更新登录令牌
     *
     * @param adminId
     * @param clientType
     * @param token
     * @return
     */
    @Override
    public boolean setToken(Long adminId, Integer clientType, String token) {
        try {
            ChannelAdminTokenDO adminTokenDO = new ChannelAdminTokenDO();
            adminTokenDO.setAdminId(adminId);
            adminTokenDO.setClientType(clientType);
            adminTokenDO.setTokenValue(token);
            adminTokenDO.setTokenCreateTime(System.currentTimeMillis());
            channelAdminTokenDAO.insertOrUpdate(adminTokenDO);
            return true;
        } catch (Exception e) {
            log.error("登录令牌更新失败", e);
        }
        return false;
    }

    /**
     * 加密令牌
     *
     * @param tokenParam
     * @return
     */
    @Override
    public String encryptToken(TokenParam tokenParam) {
        return TokenProvider.encryptToken(tokenParam);
    }

    /**
     * 解密令牌
     *
     * @param tokenEncrypted
     * @return
     */
    @Override
    public TokenParam decryptToken(String tokenEncrypted) {
        return TokenProvider.decryptToken(tokenEncrypted);
    }

    /**
     * 移除令牌
     *
     * @param adminId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean removeToken(Long adminId) {
        return channelAdminTokenDAO.deleteById(adminId) == 1;
    }

    /**
     * 修改密码
     *
     * @param adminId 管理员id
     * @param newPwd  新密码
     * @return 是否成功
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean changePwd(Long adminId, String newPwd) {
        return adminSecurityDAO.updatePassword(adminId, encryptPassword(newPwd)) == 1;
    }

    public static String encryptPassword(String password) {
        return SecureUtil.sha256(SecureUtil.md5(password).toUpperCase()).toLowerCase();
    }

    @Override
    public boolean hasPermissions(Long adminId, Long channelId) {
        if (adminId == null || channelId == null) {
            return false;
        }

        ChannelAdminInfoVO admin = chAdminService.getInfo(adminId);
        if (admin == null) {
            return false;
        }
        Long parentId = admin.getParentId();
        if (parentId == null) {
            log.error("parentId is null, 数据异常，不能获得渠道权限 admin={}", admin);
            return false;
        }

        if (parentId.equals(0L)) {
            // 顶级账号
            Long channelGroupId = admin.getChannelGroupId();
            if (channelGroupId == null) {
                log.error("channelGroupId is null, 数据异常，不能获得渠道权限 admin={}", admin);
                return false;
            }
            List<ChannelDTO> channelIds = chChannelService.getGroupChannelList(channelGroupId);
            for (ChannelDTO dto : channelIds) {
                if (dto == null) {
                    continue;
                }
                if (channelId.equals(dto.getChannelId())) {
                    return true;
                }
            }
            log.warn("顶级账号没有匹配的渠道id，不能获得渠道权限 admin={}", admin);
            return false;
        }

        // 子账号
        String authChannelIds = admin.getAuthChannelIds();
        if (StringUtils.isEmpty(authChannelIds)) {
            log.error("子账号 authChannelIds is null, 数据异常，不能获得渠道权限 admin={}", admin);
            return false;
        }
        for (String id : authChannelIds.split(",")) {
            if (NumberUtil.isLong(id)) {
                continue;
            }
            if (channelId.equals(Long.parseLong(id))) {
                return true;
            }
        }
        log.warn("子账号没有匹配的渠道id，不能获得渠道权限 admin={}", admin);
        return false;
    }

    public static void main(String[] args) {
        String password = "e10adc3949ba59abbe56e057f20f883e";
        String out = SecureUtil.sha256(SecureUtil.md5(password).toUpperCase()).toLowerCase();
        System.out.println(out);
    }
}
