package com.starbuds.server.api.channel.service.admin.impl;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.api.channel.common.pojo.admin.ChannelAdminInfoVO;
import com.starbuds.server.api.channel.common.pojo.admin.ChannelRatioReq;
import com.starbuds.server.api.channel.common.pojo.db.ChannelAdminDO;
import com.starbuds.server.api.channel.common.pojo.db.ChannelAdminSecurityDO;
import com.starbuds.server.api.channel.common.service.admin.ChAdminService;
import com.starbuds.server.api.channel.service.admin.dao.ChannelAdminDAO;
import com.starbuds.server.api.channel.service.admin.dao.ChannelAdminSecurityDAO;
import com.starbuds.server.api.channel.service.admin.dao.ChannelDAO;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.service.user.ChannelService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/1/16 13:59
 * @Description 管理员业务实现
 */
@Slf4j
@Service
public class ChAdminServiceImpl implements ChAdminService {

    @Resource
    ChannelAdminDAO channelAdminDAO;

    @Resource
    ChannelAdminSecurityDAO channelAdminSecurityDAO;

    @Reference(version = "1.0.0")
    ChannelService channelService;

    @Resource
    ChannelDAO channelDAO;

    @Override
    public ChannelAdminInfoVO getInfo(Long adminId) {
        ChannelAdminInfoVO vo = channelAdminDAO.selectVOById(adminId);
        if (vo != null) {
            vo.setChannelName(channelService.getChannelName(vo.getChannelId()));
            if (StringUtils.isNotEmpty(vo.getAuthChannelIds())) {
                Set<Long> channelIdSet = Arrays.stream(vo.getAuthChannelIds().split(",")).map(e -> Convert.toLong(e)).collect(Collectors.toSet());
                if (CollectionUtils.isEmpty(channelIdSet)) {
                    vo.setAuthChannels(new ArrayList<>());
                } else {
                    vo.setAuthChannels(channelDAO.selectChannels(channelIdSet));
                }
            }
        }
        return vo;
    }

    @Override
    public boolean isAdminExist(Long adminId) {
        return channelAdminDAO.countById(adminId) == 1;
    }

    /**
     * 创建管理员
     *
     * @param infoVO 管理员
     * @return 是否成功
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean createAdmin(ChannelAdminInfoVO infoVO) {
        ChannelAdminDO adminDO = new ChannelAdminDO();
        BeanUtils.copyProperties(infoVO, adminDO);
        if (channelAdminDAO.insert(adminDO) != 1) {
            throw new RuntimeException("创建管理员失败");
        }
        ChannelAdminSecurityDO securityDO = new ChannelAdminSecurityDO();
        securityDO.setAdminId(adminDO.getAdminId());
        securityDO.setAdminAccount(infoVO.getAdminPhone());
        securityDO.setAdminIsLocked(0);
        securityDO.setRoleId(0L);
        securityDO.setAdminPass(infoVO.getAdminPass());
        securityDO.setStatus(CommonStatus.Normal);
        if (channelAdminSecurityDAO.insert(securityDO) != 1) {
            throw new RuntimeException("保存密码失败");
        }
        return true;
    }

    /**
     * 编辑管理员
     *
     * @param adminDO 管理员
     * @return 是否成功
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean editAdmin(ChannelAdminDO adminDO) {
        return channelAdminDAO.updateById(adminDO) == 1;
    }

    /**
     * 禁用管理员
     *
     * @param adminId 管理员id
     * @return 是否成功
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean disableAdmin(Long adminId) {
        return channelAdminDAO.updateStatus(adminId, CommonStatus.Disabled) == 1;
    }

    /**
     * 启用管理员
     *
     * @param adminId 管理员id
     * @return 是否成功
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean enableAdmin(Long adminId) {
        return channelAdminDAO.updateStatus(adminId, CommonStatus.Normal) == 1;
    }

    /**
     * 删除管理员
     *
     * @param adminId 管理员id
     * @return 是否成功
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deleteAdmin(Long adminId) {
        return channelAdminDAO.updateStatus(adminId, CommonStatus.Deleted) == 1;
    }

    /**
     * 获取管理员列表
     *
     * @param adminName  管理员姓名
     * @param adminPhone 手机号
     * @param status     状态
     * @param pageIndex  当前页码
     * @param pageSize   分页大小
     * @param adminId    管理员id
     * @return result对象
     */
    @Override
    public PageList<ChannelAdminInfoVO> getAdminPageList(String adminName, String adminPhone, Integer status, int pageIndex, int pageSize, Long adminId) {
        PageList<ChannelAdminInfoVO> page = new PageList<>(pageIndex, pageSize);
        List<ChannelAdminInfoVO> list = channelAdminDAO.getAdminPageList(adminName, adminPhone, status, page.getPageCursor(), page.getPageSize(), adminId);
        if (!CollectionUtils.isEmpty(list)) {
            for (ChannelAdminInfoVO vo : list) {
                vo.setParentName(vo.getParentId() == null || vo.getParentId().equals(0L) ? "" : channelAdminDAO.selectAdminName(vo.getParentId()));
                if (StringUtils.isNotEmpty(vo.getAuthChannelIds())) {
                    Set<Long> dbChannelIdSet = Arrays.stream(vo.getAuthChannelIds().split(",")).map(e -> Convert.toLong(e)).collect(Collectors.toSet());
                    if (CollectionUtils.isEmpty(dbChannelIdSet)) {
                        vo.setAuthChannels(new ArrayList<>());
                    } else {
                        vo.setAuthChannels(channelDAO.selectChannels(dbChannelIdSet));
                    }
                }
            }
        }
        page.setList(list);
        page.setTotal(channelAdminDAO.countAdminPageList(adminName, adminPhone, status, adminId));
        return page;
    }


    /**
     * 判断是否存在账号
     *
     * @param adminPhone 手机号
     * @return 是否存在
     */
    @Override
    public boolean isPhoneExist(String adminPhone) {
        return channelAdminSecurityDAO.countByPhone(adminPhone) > 0;
    }

    @Override
    public Integer getAdminStatus(String adminPhone) {
        return channelAdminDAO.selectAdminStatus(adminPhone);
    }

    @Override
    public Long getAdminId(String adminPhone) {
        return channelAdminSecurityDAO.selectAdminId(adminPhone);
    }

    /**
     * 是否可用
     *
     * @param adminId
     * @return
     */
    @Override
    public boolean isAdminEnable(Long adminId) {
        return Optional.ofNullable(channelAdminDAO.getAdminStatus(adminId)).orElse(-1) == 1;
    }

    @Override
    public Long getChannelId(Long adminId) {
        return channelAdminDAO.selectChannelId(adminId);
    }

    @Override
    public Long getChannelGroupId(Long adminId) {
        return channelAdminDAO.selectChannelGroupId(adminId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateSecurity(ChannelAdminSecurityDO securityDO) {
        return channelAdminSecurityDAO.updateById(securityDO) == 1;
    }

    @Override
    public boolean isParent(Long adminId) {
        return channelAdminDAO.isParent(adminId) > 0;
    }

    @Override
    public boolean setChannelRatio(ChannelRatioReq ratioReq) {
        return channelDAO.updateRatio(ratioReq) > 0;
    }

    @Override
    public String getAuthChannelIds(Long adminId) {
        return channelAdminDAO.selectAuthChannelIds(adminId);
    }

    @Override
    public List<Long> getAdminIdByParentId(Long adminId) {
        return channelAdminDAO.selectAdminIdByParentId(adminId);
    }

    @Override
    public Long getParentId(Long adminId) {
        return channelAdminDAO.selectParentId(adminId);
    }

    @Override
    public ChannelAdminDO getAdmin(Long adminId) {
        return channelAdminDAO.selectAdmin(adminId);
    }

    /**
     * 获取管理员Map
     *
     * @param adminIds 管理员id集合
     * @return
     */
    @Override
    public Map<Long, ChannelAdminDO> getAdminMapByIds(Set<Long> adminIds) {
        if (CollectionUtils.isEmpty(adminIds)) {
            return Collections.EMPTY_MAP;
        }
        Map map = channelAdminDAO.selectAdminMapByIds(adminIds);
        if (CollectionUtils.isEmpty(map)) {
            return Collections.EMPTY_MAP;
        }
        return map;
    }
}
