package com.starbuds.server.api.channel.task;

import com.starbuds.server.api.channel.common.pojo.constant.NotifyApiEnum;
import com.starbuds.server.api.channel.common.pojo.qo.ChNotifyResendTempPageListQO;
import com.starbuds.server.api.channel.common.pojo.request.NotifyRequest;
import com.starbuds.server.api.channel.common.pojo.request.RedEnvelopeNotifyRequest;
import com.starbuds.server.api.channel.common.service.admin.ChChannelNotifyHisService;
import com.starbuds.server.api.channel.common.service.admin.ChChannelNotifyResendTempService;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.pojo.db.ChannelNotifyResendTempDO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.util.List;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * 渠道通知定时任务
 * 失败重发周期
 * 第一次：5秒
 * 第二次：30秒
 * 第三次：10分钟
 * 第四次：1小时
 * 第五次：24小时
 *
 * @author aoe
 * @date 2020/7/1
 */
@Component
@Slf4j
public class ChannelNotifyTask {

    private ScheduledExecutorService scheduleThreadPoll;

    /**
     * 重试周期
     */
    private long[] resendPeriods;

    @Resource
    private ChChannelNotifyResendTempService chChannelNotifyResendTempService;

    @Resource
    private ChChannelNotifyHisService chChannelNotifyHisService;

    @PostConstruct
    private void init() {
        resendPeriods = new long[]{
                5L, 30L, 10 * 60L, 60 * 60L, 24 * 60 * 60L
        };

        scheduleThreadPoll = new ScheduledThreadPoolExecutor(5, new NotifyResendThreadFactory());
        scheduleThreadPoll.scheduleAtFixedRate(new Resend(1), 0, getPeriod(1), TimeUnit.SECONDS);
        scheduleThreadPoll.scheduleAtFixedRate(new Resend(2), 0, getPeriod(2), TimeUnit.SECONDS);
        scheduleThreadPoll.scheduleAtFixedRate(new Resend(3), 0, getPeriod(3), TimeUnit.SECONDS);
        scheduleThreadPoll.scheduleAtFixedRate(new Resend(4), 0, getPeriod(4), TimeUnit.SECONDS);
        scheduleThreadPoll.scheduleAtFixedRate(new Resend(5), 0, getPeriod(5), TimeUnit.SECONDS);
    }

    private long getPeriod(int times) {
        return resendPeriods[times - 1];
    }

    /**
     * 重发通知
     */
    private class Resend implements Runnable {
        private int times;

        public Resend(int times) {
            this.times = times;
        }

        @Override
        public void run() {
            long current = System.currentTimeMillis();
            long time = current - (getPeriod(times) * 1000) - 600;
            ChNotifyResendTempPageListQO qo = new ChNotifyResendTempPageListQO();
            qo.setStartTime(time);
            qo.setEndTime(current);
            qo.setNotifyTimes(times);
            List<ChannelNotifyResendTempDO> list = chChannelNotifyResendTempService.getResendList(qo);
            if (CollectionUtils.isEmpty(list)) {
                return;
            }

            for (ChannelNotifyResendTempDO resend : list) {
                // 重发通知
                NotifyRequest request = new RedEnvelopeNotifyRequest(resend.getTradeId(), resend.getIsSuccess() == BoolIntVal.True);
                BeanUtils.copyProperties(resend, request);
                NotifyApiEnum apiEnum = NotifyApiEnum.toEnum(resend.getApiNo());
                chChannelNotifyHisService.sendNotification(resend.getChannelId(), resend.getTraceId(), apiEnum,
                        request, times + 1);
            }
            // 删除临时记录
            chChannelNotifyResendTempService.deleteResendList(qo);
        }
    }

    /**
     * The default thread factory
     */
    private static class NotifyResendThreadFactory implements ThreadFactory {
        private static final AtomicInteger poolNumber = new AtomicInteger(1);
        private final ThreadGroup group;
        private final AtomicInteger threadNumber = new AtomicInteger(1);
        private final String namePrefix;

        NotifyResendThreadFactory() {
            SecurityManager s = System.getSecurityManager();
            group = (s != null) ? s.getThreadGroup() :
                    Thread.currentThread().getThreadGroup();
            namePrefix = "pool-NotifyResend-Dispatcher" +
                    poolNumber.getAndIncrement() +
                    "-thread-";
        }

        @Override
        public Thread newThread(Runnable r) {
            Thread t = new Thread(group, r,
                    namePrefix + threadNumber.getAndIncrement(),
                    0);
            t.setDaemon(false);
            t.setPriority(Thread.NORM_PRIORITY);
            return t;
        }
    }
}
