package com.starbuds.server.api.channel.v1.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.RandomUtil;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.api.channel.common.pojo.admin.ChannelAdminInfoVO;
import com.starbuds.server.api.channel.common.pojo.admin.ChannelDTO;
import com.starbuds.server.api.channel.common.pojo.db.ChannelAdminDO;
import com.starbuds.server.api.channel.common.pojo.db.ChannelAdminSecurityDO;
import com.starbuds.server.api.channel.common.service.admin.ChAdminSecurityService;
import com.starbuds.server.api.channel.common.service.admin.ChAdminService;
import com.starbuds.server.api.channel.common.service.admin.ChChannelService;
import com.starbuds.server.api.channel.inf.PermitGuest;
import com.starbuds.server.api.channel.service.admin.impl.ChAdminSecurityServiceImpl;
import com.starbuds.server.api.channel.v1.facade.AdminFacade;
import com.starbuds.server.api.manage.common.pojo.admin.TokenParam;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.service.core.IdentityService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import javax.ws.rs.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 管理员管理
 *
 * @author jiangxiaowei
 * @date 2020/1/16 14:05
 */
@Path("/v1/admin")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class AdminFacadeImpl extends BaseFacade implements AdminFacade {

    @Autowired
    ChAdminSecurityService chAdminSecurityService;

    @Autowired
    ChAdminService chAdminService;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Resource
    ChChannelService chChannelService;

    /**
     * 账号密码登录
     *
     * @param account
     * @param password
     * @return
     */
    @POST
    @Path("login")
    @PermitGuest
    @Override
    public Result login(@FormParam("account") String account,
                        @FormParam("password") String password) {

        ChannelAdminSecurityDO adminSecurityDO = chAdminSecurityService.getSecurityByAccount(account);
        if (adminSecurityDO == null || !chAdminSecurityService.checkPass(password, adminSecurityDO.getAdminPass())) {
            return new Result(GlobalApiError.OperationFailed, "账号或密码错误");
        }

        if (!chAdminService.isAdminEnable(adminSecurityDO.getAdminId())) {
            return new Result(GlobalApiError.OperationFailed, "该账号已被停用");
        }

        // 创建令牌登录
        String token = RandomUtil.randomString(64);
        if (!chAdminSecurityService.setToken(adminSecurityDO.getAdminId(), this.getHeader().getClientType().getValue(), token)) {
            return new Result(GlobalApiError.OperationFailed, "令牌创建失败");
        }

        // 加密令牌
        TokenParam tokenParam = new TokenParam();
        tokenParam.setAdminId(adminSecurityDO.getAdminId());
        tokenParam.setRoleId(adminSecurityDO.getRoleId());
        tokenParam.setToken(token);
        tokenParam.setTimestamp(System.currentTimeMillis());
        tokenParam.setChannelId(chAdminService.getChannelId(adminSecurityDO.getAdminId()));
        String tokenEncrypted = chAdminSecurityService.encryptToken(tokenParam);

        MutableResult mResult = new MutableResult();
        mResult.put("token", tokenEncrypted);
        return mResult.build();
    }

    /**
     * 个人信息
     *
     * @return
     */
    @GET
    @Path("getInfo")
    @Override
    public Result getInfo() {
        return new Result(chAdminService.getInfo(this.getToken().getAdminId()));
    }

    /**
     * 重置密码
     *
     * @param adminId 被重置人的id
     * @param newPwd  新密码
     * @return result对象
     */
    @POST
    @Path("resetPwd")
    @Override
    public Result resetPwd(@FormParam("adminId") Long adminId,
                           @FormParam("newPwd") String newPwd) {
        if (!chAdminService.isAdminExist(adminId)) {
            return new Result(GlobalApiError.OperationFailed, "管理员不存在");
        }

        if (chAdminSecurityService.changePwd(adminId, newPwd)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }


    /**
     * 修改密码
     *
     * @param oldPwd 旧密码
     * @param newPwd 新密码
     * @return result对象
     */
    @POST
    @Path("changePwd")
    @Override
    public Result changePwd(@FormParam("oldPwd") String oldPwd,
                            @FormParam("newPwd") String newPwd) {
        ChannelAdminSecurityDO dbSecurity = chAdminSecurityService.getSecurityByAdminId(this.getToken().getAdminId());
        //校验旧密码
        if (dbSecurity == null || !chAdminSecurityService.checkPass(oldPwd, dbSecurity.getAdminPass())) {
            return new Result(GlobalApiError.OperationFailed, "旧密码不正确");
        }
        if (chAdminSecurityService.changePwd(this.getToken().getAdminId(), newPwd)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 创建管理员
     *
     * @param adminName      管理员名称
     * @param adminPhone     手机号
     * @param adminAvatar    头像
     * @param adminEmail     email
     * @param adminPass      密码
     * @param status         状态
     * @param authChannelIds 授权渠道id
     * @return result对象
     */
    @POST
    @Path("createAdmin")
    @Override
    public Result createAdmin(@FormParam("adminName") String adminName,
                              @FormParam("adminPhone") String adminPhone,
                              @FormParam("adminAvatar") String adminAvatar,
                              @FormParam("adminEmail") String adminEmail,
                              @FormParam("adminPass") String adminPass,
                              @FormParam("status") Integer status,
                              @FormParam("authChannelIds") String authChannelIds) {

        Long channelId = this.getToken().getChannelId();
        Long adminId = this.getToken().getAdminId();

        if (!chAdminService.isParent(adminId)) {
            return new Result(GlobalApiError.ServerError, "您没有权限，只有母账号才可授权渠道");
        }
        // 渠道分组id
        Long channelGroupId = chAdminService.getChannelGroupId(adminId);
        // 分组下所有渠道
        List<ChannelDTO> groupChannelList = chChannelService.getGroupChannelList(channelGroupId);
        Set<Long> channelIdSet = groupChannelList.stream().map(ChannelDTO::getChannelId).collect(Collectors.toSet());
        List<Long> channelIdList = new ArrayList<>(channelIdSet);
        if (!channelIdSet.contains(channelId)) {
            channelIdSet.add(channelId);
        }
        log.info("channelIdSet:{}", channelIdSet);
        long count = Arrays.stream(authChannelIds.split(",")).filter(e -> !channelIdSet.contains(Convert.toLong(e))).count();
        if (count > 0L) {
            return new Result(GlobalApiError.ServerError, "您只能选择管辖渠道分组下的渠道");
        }

        boolean phoneExist = chAdminService.isPhoneExist(adminPhone);
        Integer adminStatus = chAdminService.getAdminStatus(adminPhone);
        if (phoneExist) {
            if (adminStatus != CommonStatus.Deleted) {
                return new Result(GlobalApiError.OperationFailed, "该手机号已是管理员");
            } else if (adminStatus == CommonStatus.Disabled) {
                return new Result(GlobalApiError.OperationFailed, "该手机号对应的管理员被禁用，请启用即可");
            }
            ChannelAdminDO adminDO = chAdminService.getAdmin(chAdminService.getAdminId(adminPhone));
            adminDO.setAdminAvatar(adminAvatar);
            adminDO.setAdminEmail(adminEmail);
            adminDO.setAdminName(adminName);
            adminDO.setAdminStatus(status);
            adminDO.setAdminPhone(adminPhone);
            adminDO.setChannelId(CollectionUtils.isEmpty(channelIdList) ? channelId : channelIdList.get(0));
            adminDO.setParentId(adminId);
            adminDO.setAuthChannelIds(authChannelIds);
            log.info("adminDO:{}", JSON.toJSONString(adminDO));
            if (chAdminService.editAdmin(adminDO)) {
                return new Result();
            }
            // 更新密码
            ChannelAdminSecurityDO securityDO = new ChannelAdminSecurityDO();
            securityDO.setAdminAccount(adminPhone);
            securityDO.setAdminId(adminDO.getAdminId());
            securityDO.setAdminPass(ChAdminSecurityServiceImpl.encryptPassword(adminPass));
            securityDO.setStatus(CommonStatus.Normal);
            log.info("securityDO:{}", JSON.toJSONString(securityDO));
            if (chAdminService.updateSecurity(securityDO)) {
                return new Result();
            }
        } else {
            // 创建
            ChannelAdminInfoVO infoVO = new ChannelAdminInfoVO();
            infoVO.setAdminId(identityService.getId());
            infoVO.setAdminName(adminName);
            infoVO.setAdminPhone(adminPhone);
            infoVO.setAdminAvatar(adminAvatar);
            infoVO.setAdminCreateTime(System.currentTimeMillis());
            infoVO.setAdminEmail(adminEmail);
            infoVO.setAdminStatus(status);
            infoVO.setAdminPass(ChAdminSecurityServiceImpl.encryptPassword(adminPass));
            infoVO.setChannelId(CollectionUtils.isEmpty(channelIdList) ? channelId : channelIdList.get(0));
            infoVO.setParentId(adminId);
            infoVO.setAuthChannelIds(authChannelIds);
            log.info("infoVO:{}", JSON.toJSONString(infoVO));
            if (chAdminService.createAdmin(infoVO)) {
                return new Result();
            }
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取管理员列表
     *
     * @param adminName  管理员姓名
     * @param adminPhone 手机号
     * @param status     状态
     * @param pageIndex  当前页码
     * @param pageSize   分页大小
     * @return result对象
     */
    @GET
    @Path("getAdminPageList")
    @Override
    public Result getAdminPageList(@QueryParam("adminName") String adminName,
                                   @QueryParam("adminPhone") String adminPhone,
                                   @QueryParam("status") Integer status,
                                   @QueryParam("pageIndex") int pageIndex,
                                   @QueryParam("pageSize") int pageSize) {
        Long adminId = this.getToken().getAdminId();
        return new Result(chAdminService.getAdminPageList(adminName, adminPhone, status, pageIndex, pageSize, adminId));
    }

    /**
     * 编辑管理员
     *
     * @param adminId        管理员id
     * @param adminName      管理员名称
     * @param adminAvatar    头像
     * @param adminEmail     email
     * @param status         状态
     * @param authChannelIds 授权渠道id
     * @return result对象
     */
    @POST
    @Path("editAdmin")
    @Override
    public Result editAdmin(@FormParam("adminId") Long adminId,
                            @FormParam("adminName") String adminName,
                            @FormParam("adminAvatar") String adminAvatar,
                            @FormParam("adminEmail") String adminEmail,
                            @FormParam("status") Integer status,
                            @FormParam("authChannelIds") String authChannelIds) {
        Long channelId = this.getToken().getChannelId();
        Long curAdminId = this.getToken().getAdminId();
        log.info("channelId:{},curAdminId:{}", channelId, curAdminId);

        if (!chAdminService.isAdminExist(adminId)) {
            return new Result(GlobalApiError.OperationFailed, "管理员不存在");
        }

        if (!chAdminService.isParent(curAdminId)) {
            return new Result(GlobalApiError.ServerError, "您没有权限，只有母账号才可授权渠道");
        }
        // 渠道分组id
        Long channelGroupId = chAdminService.getChannelGroupId(curAdminId);
        // 分组下所有渠道
        List<ChannelDTO> groupChannelList = chChannelService.getGroupChannelList(channelGroupId);
        Set<Long> channelIdSet = groupChannelList.stream().map(ChannelDTO::getChannelId).collect(Collectors.toSet());
        List<Long> channelIdList = new ArrayList<>(channelIdSet);
        if (!channelIdSet.contains(channelId)) {
            channelIdSet.add(channelId);
        }
        log.info("channelIdSet:{},authChannelIds:{}", channelIdSet, authChannelIds);
        long count = Arrays.stream(authChannelIds.split(",")).filter(e -> !channelIdSet.contains(Convert.toLong(e))).count();
        if (count > 0) {
            return new Result(GlobalApiError.ServerError, "您只能选择管辖渠道分组下的渠道");
        }

        ChannelAdminDO adminDO = chAdminService.getAdmin(adminId);
        if (adminDO != null && !curAdminId.equals(adminDO.getParentId())) {
            return new Result(GlobalApiError.ServerError, "您操作的数据没权限");
        }
        adminDO.setAdminName(adminName);
        adminDO.setAdminAvatar(adminAvatar);
        adminDO.setAdminEmail(adminEmail);
        adminDO.setAdminStatus(status);
        adminDO.setChannelId(CollectionUtils.isEmpty(channelIdList) ? channelId : channelIdList.get(0));
        adminDO.setAuthChannelIds(authChannelIds);
        if (chAdminService.editAdmin(adminDO)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 禁用管理员
     *
     * @param adminId 管理员id
     * @return result对象
     */
    @POST
    @Path("disableAdmin")
    @Override
    public Result disableAdmin(@FormParam("adminId") Long adminId) {
        if (!chAdminService.isAdminExist(adminId)) {
            return new Result(GlobalApiError.OperationFailed, "管理员不存在");
        }
        if (chAdminService.disableAdmin(adminId)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 启用管理员
     *
     * @param adminId 管理员id
     * @return result对象
     */
    @POST
    @Path("enableAdmin")
    @Override
    public Result enableAdmin(@FormParam("adminId") Long adminId) {
        if (!chAdminService.isAdminExist(adminId)) {
            return new Result(GlobalApiError.OperationFailed, "管理员不存在");
        }
        if (chAdminService.enableAdmin(adminId)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 删除管理员
     *
     * @param adminId 管理员id
     * @return result对象
     */
    @POST
    @Path("deleteAdmin")
    @Override
    public Result deleteAdmin(@FormParam("adminId") Long adminId) {
        if (!chAdminService.isAdminExist(adminId)) {
            return new Result(GlobalApiError.OperationFailed, "管理员不存在");
        }
        if (chAdminService.deleteAdmin(adminId)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

}
