package com.starbuds.server.api.channel.v1.impl;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.api.channel.common.pojo.admin.ChannelAppReq;
import com.starbuds.server.api.channel.common.pojo.admin.ChannelDTO;
import com.starbuds.server.api.channel.common.pojo.admin.ChannelRatioReq;
import com.starbuds.server.api.channel.common.service.admin.ChAdminSecurityService;
import com.starbuds.server.api.channel.common.service.admin.ChAdminService;
import com.starbuds.server.api.channel.common.service.admin.ChChannelService;
import com.starbuds.server.api.channel.v1.facade.ChannelFacade;
import com.starbuds.server.api.manage.common.pojo.admin.TokenParam;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.db.ChannelDO;
import com.starbuds.server.common.service.user.ChannelService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.ws.rs.*;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 渠道管理
 *
 * @author jiangxiaowei
 * @date 2020/4/28 14:40
 */
@Path("/v1/channel")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class ChannelFacadeImpl extends BaseFacade implements ChannelFacade {

    @Resource
    ChChannelService chChannelService;

    @Resource
    ChAdminService chAdminService;

    @Reference(version = "1.0.0")
    ChannelService channelService;

    @Resource
    private ChAdminSecurityService chAdminSecurityService;

    /**
     * 获取渠道信息
     *
     * @param channelId
     * @return
     */
    @GET
    @Path("getInfo")
    @Override
    public Result getInfo(@QueryParam("channelId") Long channelId) {
        if (channelId == null) {
            channelId = this.getToken().getChannelId();
        } else {
            Long adminId = this.getToken().getAdminId();
            // 母账号
            if (chAdminService.isParent(adminId)) {
                Long channelGroupId = chAdminService.getChannelGroupId(adminId);
                List<ChannelDTO> dtoList = chChannelService.getGroupChannelList(channelGroupId);
                Set<Long> channelIdSet = dtoList.stream().map(ChannelDTO::getChannelId).collect(Collectors.toSet());
                channelIdSet.add(channelId);
                if (!channelId.equals(this.getToken().getChannelId()) && !channelIdSet.contains(channelId)) {
                    return new Result(GlobalApiError.ServerError, "只能查看自己管辖的渠道");
                }
            } else {
                // 子账号
                String authChannelIds = chAdminService.getAuthChannelIds(adminId);
                if (StringUtils.isNotEmpty(authChannelIds)) {
                    Set<Long> authChannelIdSet = Arrays.stream(authChannelIds.split(",")).map(e -> Convert.toLong(e)).collect(Collectors.toSet());
                    if (!channelId.equals(this.getToken().getChannelId()) && !authChannelIdSet.contains(channelId)) {
                        return new Result(GlobalApiError.ServerError, "只能查看自己管辖的渠道");
                    }
                }
            }
        }
        return new Result(chChannelService.getInfo(channelId));
    }

    /**
     * 编辑渠道 App
     *
     * @param channelReq
     * @return
     */
    @Path("editChannelApp")
    @POST
    @Override
    public Result editChannelApp(@BeanParam ChannelAppReq cAppReq) {
        TokenParam token = getToken();
        if (token == null) {
            return new Result(GlobalApiError.OperationFailed, "请登录");
        }
        Long channelId = getChannelId(cAppReq.getChannelId());
        if (!chAdminSecurityService.hasPermissions(token.getAdminId(), channelId)) {
            log.warn("您没有这个渠道的权限 adminId={} channelId={}", token.getAdminId(), channelId);
            return new Result(GlobalApiError.BadRequest, "您没有这个渠道的权限");
        }
        if (channelId == null) {
            return new Result(GlobalApiError.OperationFailed, "channelId 不存在");
        }
        if (!chChannelService.existByChannelId(channelId)) {
            return new Result(GlobalApiError.OperationFailed, "渠道不存在");
        }

        ChannelDO channelDO = new ChannelDO();
        channelDO.setChannelId(channelId);
        BeanUtils.copyProperties(cAppReq, channelDO);
        int updateCount = chChannelService.update(channelDO);
        if (updateCount == 1) {
            log.info("更新渠道信息成功 channelDO={}", channelDO);
            return new Result();
        }

        log.warn("更新渠道信息失败 updateCount={} channelDO={}", updateCount, channelDO);
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获得渠道 App 信息
     *
     * @return
     */
    @GET
    @Path("getChannelApp")
    @Override
    public Result getChannelApp(@QueryParam("channelId") Long channelIdReq) {
        TokenParam token = getToken();
        if (token == null) {
            return new Result(GlobalApiError.OperationFailed, "请登录");
        }
        Long channelId = getChannelId(channelIdReq);
        if (!chAdminSecurityService.hasPermissions(token.getAdminId(), channelId)) {
            log.warn("您没有这个渠道的权限 adminId={} channelId={}", token.getAdminId(), channelId);
            return new Result(GlobalApiError.BadRequest, "您没有这个渠道的权限");
        }
        if (channelId == null) {
            return new Result(GlobalApiError.OperationFailed, "channelId 不存在");
        }
        if (!chChannelService.existByChannelId(channelId)) {
            return new Result(GlobalApiError.OperationFailed, "渠道不存在");
        }
        return new Result(chChannelService.getChannelApp(channelId));
    }

    /**
     * 生成渠道 APP 秘钥
     *
     * @return
     */
    @Path("createSecretKey")
    @POST
    @Override
    public Result createSecretKey(@FormParam("channelId") Long channelIdReq) {
        TokenParam token = getToken();
        if (token == null) {
            return new Result(GlobalApiError.OperationFailed, "请登录");
        }
        Long channelId = getChannelId(channelIdReq);
        if (!chAdminSecurityService.hasPermissions(token.getAdminId(), channelId)) {
            log.warn("您没有这个渠道的权限 adminId={} channelId={}", token.getAdminId(), channelId);
            return new Result(GlobalApiError.BadRequest, "您没有这个渠道的权限");
        }
        if (channelId == null) {
            return new Result(GlobalApiError.OperationFailed, "channelId 不存在");
        }
        if (!chChannelService.existByChannelId(channelId)) {
            return new Result(GlobalApiError.OperationFailed, "渠道不存在");
        }

        int keyCount = chChannelService.getSecretKeyCountByChannelId(channelId);
        if (keyCount > 1) {
            return new Result(GlobalApiError.OperationFailed, "最多创建2个秘钥");
        }

        String key = chChannelService.createSecretKey(channelId);
        if (key == null) {
            return new Result(GlobalApiError.OperationFailed, "创建失败");
        }

        return new Result(key);
    }

    /**
     * 变更渠道APP秘钥
     *
     * @param secretId
     * @return
     */
    @Path("changeSecretKey")
    @POST
    @Override
    public Result changeSecretKey(@FormParam("secretId") Long secretId, @FormParam("channelId") Long channelIdReq) {
        if (secretId == null) {
            return new Result(GlobalApiError.OperationFailed, "secretId 必传");
        }
        TokenParam token = getToken();
        if (token == null) {
            return new Result(GlobalApiError.OperationFailed, "请登录");
        }
        Long channelId = getChannelId(channelIdReq);
        if (!chAdminSecurityService.hasPermissions(token.getAdminId(), channelId)) {
            log.warn("您没有这个渠道的权限 adminId={} channelId={}", token.getAdminId(), channelId);
            return new Result(GlobalApiError.BadRequest, "您没有这个渠道的权限");
        }
        if (channelId == null) {
            return new Result(GlobalApiError.OperationFailed, "channelId 不存在");
        }
        if (!chChannelService.existByChannelId(channelId)) {
            return new Result(GlobalApiError.OperationFailed, "渠道不存在");
        }

        String key = chChannelService.changeSecretKey(channelId, secretId);
        if (key == null) {
            return new Result(GlobalApiError.OperationFailed, "更新失败");
        }

        return new Result(key);
    }

    /**
     * 删除渠道APP秘钥
     *
     * @param secretId
     * @return
     */
    @Path("deleteSecretKey")
    @POST
    @Override
    public Result deleteSecretKey(@FormParam("secretId") Long secretId, @FormParam("channelId") Long channelIdReq) {
        if (secretId == null) {
            return new Result(GlobalApiError.OperationFailed, "secretId 必传");
        }
        TokenParam token = getToken();
        if (token == null) {
            return new Result(GlobalApiError.OperationFailed, "请登录");
        }
        Long channelId = getChannelId(channelIdReq);
        if (!chAdminSecurityService.hasPermissions(token.getAdminId(), channelId)) {
            log.warn("您没有这个渠道的权限 adminId={} channelId={}", token.getAdminId(), channelId);
            return new Result(GlobalApiError.BadRequest, "您没有这个渠道的权限");
        }
        if (channelId == null) {
            return new Result(GlobalApiError.OperationFailed, "channelId 不存在");
        }
        if (!chChannelService.existByChannelId(channelId)) {
            return new Result(GlobalApiError.OperationFailed, "渠道不存在");
        }

        int keyCount = chChannelService.getSecretKeyCountByChannelId(channelId);
        if (keyCount < 2) {
            return new Result(GlobalApiError.OperationFailed, "至少保留1个秘钥");
        }

        int updateCount = chChannelService.deleteSecretKey(channelId, secretId);
        if (updateCount == 1) {
            log.info("删除秘钥成功 secretId={} isActivate={}", secretId);
            return new Result();
        }

        log.warn("删除秘钥失败 secretId={} isActivate={}", secretId);
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 启用/禁用渠道APP秘钥
     *
     * @param secretId
     * @param isActivate 1:启用; 0:禁用
     * @return
     */
    @Path("activateSecretKey")
    @POST
    @Override
    public Result activateSecretKey(@FormParam("secretId") Long secretId, @FormParam("isActivate") Integer isActivate,
                                    @FormParam("channelId") Long channelIdReq) {
        if (secretId == null) {
            return new Result(GlobalApiError.OperationFailed, "secretId 必传");
        }
        if (isActivate == null) {
            return new Result(GlobalApiError.OperationFailed, "isActivate 必传");
        }
        if (isActivate != BoolIntVal.True && isActivate != BoolIntVal.False) {
            return new Result(GlobalApiError.OperationFailed, "isActivate 的值无效");
        }
        TokenParam token = getToken();
        if (token == null) {
            return new Result(GlobalApiError.OperationFailed, "请登录");
        }
        Long channelId = getChannelId(channelIdReq);
        if (!chAdminSecurityService.hasPermissions(token.getAdminId(), channelId)) {
            log.warn("您没有这个渠道的权限 adminId={} channelId={}", token.getAdminId(), channelId);
            return new Result(GlobalApiError.BadRequest, "您没有这个渠道的权限");
        }
        if (channelId == null) {
            return new Result(GlobalApiError.OperationFailed, "channelId 不存在");
        }
        if (!chChannelService.existByChannelId(channelId)) {
            return new Result(GlobalApiError.OperationFailed, "渠道不存在");
        }

        if (isActivate == BoolIntVal.False) {
            return new Result(GlobalApiError.OperationFailed, "选择一个秘钥为启用状态，其他自动变为禁用");
        }
        int updateCount = chChannelService.activateSecretKey(channelId, secretId, isActivate);
        if (updateCount == 1) {
            log.info("更新激活状态成功 secretId={} isActivate={}", secretId, isActivate);
            return new Result();
        }

        log.warn("更新激活状态失败 secretId={} isActivate={}", secretId, isActivate);
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 查询（自己渠道下）所有未删除的秘钥
     *
     * @return
     */
    @GET
    @Path("getSecretKeyList")
    @Override
    public Result getSecretKeyList(@QueryParam("channelId") Long channelIdReq) {
        TokenParam token = getToken();
        if (token == null) {
            return new Result(GlobalApiError.OperationFailed, "请登录");
        }
        Long channelId = getChannelId(channelIdReq);
        if (!chAdminSecurityService.hasPermissions(token.getAdminId(), channelId)) {
            log.warn("您没有这个渠道的权限 adminId={} channelId={}", token.getAdminId(), channelId);
            return new Result(GlobalApiError.BadRequest, "您没有这个渠道的权限");
        }
        if (channelId == null) {
            return new Result(GlobalApiError.OperationFailed, "channelId 不存在");
        }
        if (!chChannelService.existByChannelId(channelId)) {
            return new Result(GlobalApiError.OperationFailed, "渠道不存在");
        }

        return new Result(chChannelService.getSecretKeyList(channelId));
    }

    /**
     * 获取渠道分组下的渠道集合
     *
     * @return
     */
    @GET
    @Path("getChannelListByGroupId")
    @Override
    public Result getChannelListByGroupId() {
        MutableResult mResult = new MutableResult();

        Long adminId = this.getToken().getAdminId();
        Long channelId = this.getToken().getChannelId();

        List<ChannelDTO> channelList = new ArrayList<>();
        // 渠道分组id
        Long channelGroupId = chAdminService.getChannelGroupId(adminId);
        String authChannelIds = chAdminService.getAuthChannelIds(adminId);
        // 母账号
        if (chAdminService.isParent(adminId)) {
            // 分组下所有渠道
            List<ChannelDTO> groupChannelList = chChannelService.getGroupChannelList(channelGroupId);
            Set<Long> channelIdSet = groupChannelList.stream().map(ChannelDTO::getChannelId).collect(Collectors.toSet());
            if (!channelIdSet.contains(channelId)) {
                // 当前渠道
                ChannelDTO dto = new ChannelDTO();
                dto.setChannelId(channelId);
                dto.setChannelName(channelService.getChannelName(dto.getChannelId()));
                groupChannelList.add(0, dto);
            }
            channelList.addAll(groupChannelList);
        } else {
            if (StringUtils.isNotEmpty(authChannelIds)) {
                Set<Long> authChannelIdSet = Arrays.stream(authChannelIds.split(",")).map(e -> Convert.toLong(e)).collect(Collectors.toSet());
                if (!authChannelIdSet.contains(channelId)) {
                    authChannelIdSet.add(channelId);
                }
                List<ChannelDTO> authChannels = chChannelService.getChannelList(authChannelIdSet);
                channelList.addAll(authChannels);
            }
        }

        mResult.put("list", channelList);
        mResult.put("channelGroupId", channelGroupId);
        mResult.put("authChannelIds", authChannelIds);
        return mResult.build();
    }

    /**
     * 设置渠道分成比例
     *
     * @param ratioReq
     * @return
     */
    @POST
    @Path("setChannelRatio")
    @Override
    public Result setChannelRatio(@BeanParam ChannelRatioReq ratioReq) {
        Long adminId = this.getToken().getAdminId();
        Long channelId = this.getToken().getChannelId();
        if (ratioReq.getChannelId() == null) {
            return new Result(GlobalApiError.ServerError, "请选择渠道");
        }
        if (ratioReq.getTodayInstallRatio() == null || ratioReq.getTodayInstallRatio() < 0D || ratioReq.getTodayInstallRatio() > 1D) {
            return new Result(GlobalApiError.ServerError, "今日安装比例非法");
        }
        if (ratioReq.getTodaySplitRatio() == null || ratioReq.getTodaySplitRatio() < 0D || ratioReq.getTodaySplitRatio() > 1D) {
            return new Result(GlobalApiError.ServerError, "今日分成比例非法");
        }
        if (ratioReq.getTomorrowInstallRatio() == null || ratioReq.getTomorrowInstallRatio() < 0D || ratioReq.getTomorrowInstallRatio() > 1D) {
            return new Result(GlobalApiError.ServerError, "明日安装比例非法");
        }
        if (ratioReq.getTomorrowSplitRatio() == null || ratioReq.getTomorrowSplitRatio() < 0D || ratioReq.getTomorrowSplitRatio() > 1D) {
            return new Result(GlobalApiError.ServerError, "明日分成比例非法");
        }
        if (!chAdminService.isParent(adminId)) {
            return new Result(GlobalApiError.ServerError, "只有母账号才可以设置");
        }
        Long channelGroupId = chAdminService.getChannelGroupId(adminId);
        List<ChannelDTO> dtoList = chChannelService.getGroupChannelList(channelGroupId);
        Set<Long> channelIdSet = dtoList.stream().map(ChannelDTO::getChannelId).collect(Collectors.toSet());
        channelIdSet.add(channelId);
        if (!channelIdSet.contains(ratioReq.getChannelId())) {
            return new Result(GlobalApiError.ServerError, "只能查看自己管辖的渠道");
        }

        ratioReq.setChannelId(ratioReq.getChannelId());

        if (chAdminService.setChannelRatio(ratioReq)) {
            return new Result();
        }
        return new Result(GlobalApiError.ServerError);
    }

    /**
     * 获取渠道列表
     *
     * @param channelName
     * @param status
     * @param pageIndex
     * @param pageSize
     * @return
     */
    @GET
    @Path("getChannelPageList")
    @Override
    public Result getChannelPageList(@QueryParam("channelName") String channelName,
                                     @QueryParam("status") Integer status,
                                     @QueryParam("pageIndex") int pageIndex,
                                     @QueryParam("pageSize") int pageSize) {
        Long adminId = this.getToken().getAdminId();
        Long channelId = this.getToken().getChannelId();
        Set<Long> channelIdSet = new HashSet<>();
        channelIdSet.add(channelId);
        // 母账号
        if (chAdminService.isParent(adminId)) {
            Long channelGroupId = chAdminService.getChannelGroupId(this.getToken().getAdminId());
            List<ChannelDTO> dtoList = chChannelService.getGroupChannelList(channelGroupId);
            channelIdSet = dtoList.stream().map(ChannelDTO::getChannelId).collect(Collectors.toSet());
        } else {
            // 子账号
            String authChannelIds = chAdminService.getAuthChannelIds(adminId);
            if (StringUtils.isNotEmpty(authChannelIds)) {
                Set<Long> authChannelIdSet = Arrays.stream(authChannelIds.split(",")).map(e -> Convert.toLong(e)).collect(Collectors.toSet());
                channelIdSet.addAll(authChannelIdSet);
            }
        }
        return new Result(chChannelService.getChannelPageList(channelIdSet, channelName, status, pageIndex, pageSize));
    }
}
