package com.starbuds.server.api.channel.v1.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.api.channel.common.pojo.bo.RedEnvelopeDetail;
import com.starbuds.server.api.channel.common.pojo.db.ChannelSecretRsaDO;
import com.starbuds.server.api.channel.common.pojo.facade.OpenApiUserInfoRes;
import com.starbuds.server.api.channel.common.pojo.form.OpenApiRedEnvelopeForm;
import com.starbuds.server.api.channel.common.pojo.vo.ChannelWalletVO;
import com.starbuds.server.api.channel.common.service.admin.ChChannelRedEnvelopeHisService;
import com.starbuds.server.api.channel.common.service.admin.ChChannelSecretRsaService;
import com.starbuds.server.api.channel.common.service.admin.ChChannelService;
import com.starbuds.server.api.channel.common.service.admin.ChChannelWalletService;
import com.starbuds.server.api.channel.inf.OpenApi;
import com.starbuds.server.api.channel.service.admin.dao.ChannelSecretDAO;
import com.starbuds.server.api.channel.util.ChannelOpenApiSingUtil;
import com.starbuds.server.api.channel.v1.facade.OpenApiFacade;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.db.ChannelRedEnvelopeHisDO;
import com.starbuds.server.common.pojo.db.UserDO;
import com.starbuds.server.common.pojo.finance.ChannelExchangeOrder;
import com.starbuds.server.common.pojo.user.TokenBO;
import com.starbuds.server.common.service.user.SecurityService;
import com.starbuds.server.common.service.user.UserService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.ws.rs.FormParam;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import java.util.*;

import static com.starbuds.server.common.utils.CheckFormUtil.checkCreate;

/**
 * 开放接口实现
 *
 * @author aoe
 * @date 2020/6/24
 */
@Path("/v1/openapi")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class OpenApiFacadeImpl extends BaseFacade implements OpenApiFacade {

    @Reference(version = "1.0.0")
    private SecurityService securityService;

    @Reference(version = "1.0.0")
    private UserService userService;

    @Resource
    private ChannelSecretDAO channelSecretDAO;

    @Resource
    private ChChannelService chChannelService;

    @Resource
    private ChChannelSecretRsaService chChannelSecretRsaService;

    @Resource
    private ChChannelRedEnvelopeHisService chChannelRedEnvelopeHisService;

    @Resource
    private ChChannelWalletService chChannelWalletService;

    /**
     * 获取星芽用户信息
     *
     * @param token     星芽token
     * @param channelId 渠道id
     * @return
     */
    @Override
    @OpenApi
    @POST
    @Path("getUser")
    public Result getUser(@FormParam("token") String token, @FormParam("channelId") Long channelId) {
        if (StringUtils.isBlank(token) || channelId == null) {
            return new Result(GlobalApiError.BadRequest);
        }
        String secretKey = channelSecretDAO.selectActivateSecretKey(channelId);
        if (secretKey == null) {
            return new Result(GlobalApiError.OperationFailed, "没有匹配的秘钥");
        }

        TokenBO tokenBO = securityService.getToken(token);
        if (tokenBO == null || tokenBO.getUserId() == null) {
            return new Result(GlobalApiError.OperationFailed, "令牌无效");
        }

        UserDO userDO = userService.getUserById(tokenBO.getUserId());
        if (userDO == null) {
            return new Result(GlobalApiError.OperationFailed, "用户不存在");
        }

        OpenApiUserInfoRes user = new OpenApiUserInfoRes();
        BeanUtils.copyProperties(userDO, user);
        user.setPhone(userDO.getUserPhone());
        user.setNickname(userDO.getUserName());
        user.setUserId("" + userDO.getUserId());

        return new Result(user);
    }

    /**
     * 发红包
     *
     * @param data
     * @return
     */
    @Override
    @OpenApi
    @POST
    @Path("sendRedEnvelope")
    public Result sendRedEnvelope(@FormParam("data") String data) {
        OpenApiRedEnvelopeForm form = JSONObject.parseObject(data, OpenApiRedEnvelopeForm.class);
        String check = checkCreate(form);
        if (check != null) {
            return new Result(GlobalApiError.BadRequest, check);
        }
        if (form.getCoinType() - WalletTypeEnum.STAR_DIAMOND.getValue() != 0) {
            return new Result(GlobalApiError.BadRequest, "无效的coinType");
        }
        List<RedEnvelopeDetail> detailList = form.getDetailList();
        if (CollectionUtil.isEmpty(detailList)) {
            return new Result(GlobalApiError.BadRequest, "detailList不能为空");
        }
        if (detailList.size() > 100) {
            return new Result(GlobalApiError.BadRequest, "最多包含100条红包明细");
        }

        Long channelId = form.getChannelId();
        // 验签
        ChannelSecretRsaDO rsa = chChannelSecretRsaService.getRsa(channelId);
        if (rsa == null) {
            return new Result(GlobalApiError.BadRequest, "请配置渠道RSA信息");
        }
        String pubKey = rsa.getPubKey();
        if (StringUtils.isBlank(pubKey)) {
            return new Result(GlobalApiError.BadRequest, "请上传渠道RSA公钥");
        }
        try {
            boolean flag = ChannelOpenApiSingUtil.verifyFromJson(pubKey, form.getSign(), data);
            if (!flag) {
                return new Result(GlobalApiError.BadRequest, "无效的签名");
            }
        } catch (Exception e) {
            log.error("发红包的RSA验签失败 " + data, e);
            return new Result(GlobalApiError.BadRequest, "验签失败,无效的签名");
        }
        if (!chChannelService.existByChannelId(channelId)) {
            return new Result(GlobalApiError.BadRequest, "channelId无效");
        }
        // 检查余额
        ChannelWalletVO wallet = chChannelWalletService.getWalletVO(channelId);
        if (wallet == null) {
            return new Result(GlobalApiError.BadRequest, "余额不足");
        }
        Long totalAmount = chChannelRedEnvelopeHisService.getTotalAmount(detailList);
        if (totalAmount > wallet.getBalance()) {
            return new Result(GlobalApiError.BadRequest, "余额不足");
        }

        // 验证兑换明细
        // key:区号; value: 手机号
        Map<String, Set<String>> areaCodePhoneMap = new HashMap<>();
        Set<Long> userIdSet = new HashSet<>();
        for (RedEnvelopeDetail detail : detailList) {
            Long userId = detail.getUserId();
            String phone = detail.getPhone();
            Long amount = detail.getAmount();

            if (userId == null && StringUtils.isBlank(phone)) {
                return new Result(GlobalApiError.BadRequest, "userId、phone不能同时为空");
            }
            if (amount == null) {
                return new Result(GlobalApiError.BadRequest, "amount不能为空");
            }
            if (amount < 1) {
                return new Result(GlobalApiError.BadRequest, "红包金额必须大于0");
            }

            if (userId != null) {
                userIdSet.add(userId);
            }
            if (StringUtils.isNotBlank(phone)) {
                String areaCode = detail.getAreaCode();
                if (StringUtils.isBlank(areaCode)) {
                    areaCode = "86";
                }
                Set<String> phoneSet = areaCodePhoneMap.get(areaCode);
                if (phoneSet == null) {
                    phoneSet = new HashSet<>();
                    areaCodePhoneMap.put(areaCode, phoneSet);
                }
                phoneSet.add(phone);
            }
        }
        // key:areaCode-phone; value:userId
        Map<String, Long> phoneMap = new HashMap<>();
        int phoneCount = 0;
        for (Map.Entry<String, Set<String>> entry : areaCodePhoneMap.entrySet()) {
            String areaCode = entry.getKey();
            phoneCount += entry.getValue().size();

            Map<String, Long> map = userService.getPhoneUserIdMap(entry.getValue(), areaCode);
            for (Map.Entry<String, Long> phoneEntry : map.entrySet()) {
                String key = StrUtil.format("{}-{}", areaCode, phoneEntry.getKey());
                phoneMap.put(key, phoneEntry.getValue());
            }
        }
        if (phoneMap.size() - phoneCount != 0) {
            return new Result(GlobalApiError.BadRequest, "存在无效的手机号");
        }

        // 手机号转userId
        Map<Long, Long> userAmountMap = new HashMap<>();
        for (RedEnvelopeDetail detail : detailList) {
            Long userId = detail.getUserId();
            if (userId == null) {
                String areaCode = detail.getAreaCode();
                if (StringUtils.isBlank(areaCode)) {
                    areaCode = "86";
                }
                String key = StrUtil.format("{}-{}", areaCode, detail.getPhone());
                userId = phoneMap.get(key);
            }
            Long amount = userAmountMap.get(userId);
            if (amount == null) {
                amount = 0L;
            }
            // 合并同一个人的兑换金额
            userAmountMap.put(userId, amount + detail.getAmount());
        }
        int userCount = userService.getCountByUserIds(userAmountMap.keySet());
        if (userAmountMap.size() - userCount != 0) {
            return new Result(GlobalApiError.BadRequest, "存在无效的用户id");
        }

        ChannelRedEnvelopeHisDO po = new ChannelRedEnvelopeHisDO();
        BeanUtils.copyProperties(form, po);
        po.setReceive(data);
        po.setIp(getIpAddress());
        po.setAdminId(0L);
        po.setCoinType(WalletTypeEnum.STAR_DIAMOND.getValue());

        List<ChannelExchangeOrder> exchangeOrders = new ArrayList<>(userAmountMap.size());
        for (Map.Entry<Long, Long> entry : userAmountMap.entrySet()) {
            ChannelExchangeOrder order = new ChannelExchangeOrder();
            order.setUserId(entry.getKey());
            order.setAmount(entry.getValue());
            exchangeOrders.add(order);
        }
        ChannelRedEnvelopeHisDO poNew = null;
        try {
            poNew = chChannelRedEnvelopeHisService.saveRedEnvelope(po, detailList, exchangeOrders);
        } catch (Exception e) {
            log.error("发红包请求接收失败 " + data, e);
        }
        if (poNew == null) {
            return new Result(GlobalApiError.OperationFailed, "发红包请求接收失败");
        }
        chChannelRedEnvelopeHisService.asyncSendDiamond(channelId, po.getHisId(), exchangeOrders, poNew.getTradeId());
        return new Result("发红包请求接收成功");
    }
}
